/* Copyright 1996, Jan D. Wolter, All Rights Reserved. */

#include "backtalk.h"
#include "sysdict.h"

#include "str.h"
#include "event.h"
#include "exec.h"

/* WARNING:  Entries with this table must match up with indexes in
 * incl/event.h
 */

struct evtbl {
    int event_var;
    char event_name[12];
    char argtype[12];
    } events[] = {
	{VAR_ONPOST,	"post" },	/* EVENT_POST */
	{VAR_ONPOST,	"link" },	/* EVENT_LINK */
	{VAR_ONRESPONSE,"response" },	/* EVENT_RESPONSE */
	{VAR_ONITEM,	"retitle" },	/* EVENT_RETITLE */
	{VAR_ONITEM,	"kill" },	/* EVENT_KILL */
	{VAR_ONITEM,	"freeze" },	/* EVENT_FREEZE */
	{VAR_ONITEM,	"retire" },	/* EVENT_RETIRE */
	{VAR_ONRESP,	"edit" },	/* EVENT_EDIT */
	{VAR_ONRESP,	"erase" },	/* EVENT_ERASE */
	{VAR_ONRESP,	"hide" },	/* EVENT_HIDE */
    };


/* TRIGGER(event_code, args...)
 *
 *   Trigger a conference event.  If there are any event handlers defined on
 *   the given event code, fork a child to run each of them.  Each
 *   child process starts with a copy of dictionary content, but the stack
 *   contains only the given args followed by the event type name from the
 *   above table.  The args array is a string of "I" and "C" characters
 *   indicating the type of the remaining arguments, which are all pushed
 *   on the stack before the type.  "I" is integer, "C" is character in static
 *   memory (it will not be freed by this).  Each child returns true, the
 *   parent returns false.
 */

int trigger(int code, char *args, ...)
{
    va_list ap;
    char *p, *h, ch;
    char *handlers;
    char script[70]= "event/";

    handlers= sdict(events[code].event_var);

    for (h= handlers; h != NULL && *h != '\0'; h= p)
    {
	p= firstin(h,",");
	if (p-h == 0 || p-h > 50) continue;
	strncpy(script+6, h, p-h);

	if (fork() == 0)
	{
	    /* Child process setup */
	    initchild(script);

	    /* Set up stack */
	    func_clear();
	    if (args != NULL)
	    {
		va_start(ap,args);
		for (;;)
		{
		    switch (*(args++))
		    {
			case 'C':
			    push_string(va_arg(ap, char *),TRUE);
			    break;
			case 'I':
			    push_integer(va_arg(ap, int));
			    break;
			case '\0':
			    goto done;
		    }
		}
    done:	    va_end(ap);
	    }

	    /* Push the event type name */
	    push_string(events[code].event_name, TRUE);

	    /* Child runs with admin powers */
	    seti(VAR_AMADM, 1);

	    return 1;
	}

    }
    return 0;
}
